/* See LICENSE file for copyright and license details. */

/* appearance */
static const unsigned int borderpx  = 2;        /* border pixel of windows */
static const unsigned int snap      = 32;       /* snap pixel */
static const int showbar            = 1;        /* 0 means no bar */
static const int topbar             = 1;        /* 0 means bottom bar */
/* static const char *fonts[]          = { "dina:size=9:antialias=false" }; */
/* static const char *fonts[]          = { "terminus:size=12" }; */
static const char *fonts[]          = { "pragmatapro:size=11" };
static const char col_gray1[] = "#000000";
static const char col_gray2[] = "#444444";
static const char col_gray3[] = "#ffffff";
static const char col_gray4[] = "#1d1f21";
static const char *colors[][3]      = {
	/*               fg         bg         border   */
	[SchemeNorm] = { col_gray3, col_gray1, col_gray1 },
	[SchemeSel]  = { col_gray1, col_gray3, col_gray3 },
};

/* tagging */
static const char *tags[] = { "1", "2", "3", "4", "5", "6", "7", "8", "9" };
/* static const char *tags[] = { "一", "二", "三", "四", "五", "六", "七", "八", "九" }; */


/* floatpos:
 *    1--2--3
 *    4--5--6
 *    7--8--9 
 * 0: disabled */
static const Rule rules[] = {
	/* xprop(1):
	 *	WM_CLASS(STRING) = instance, class
	 *	WM_NAME(STRING) = title
	 */
	/* class         instance      title    tags   floatpos  isfloating  monitor */
	{ "Tor Browser", NULL,         NULL,    0,     5,        1,          -1 },
	{ "tabbed",      NULL,         NULL,    1<<1,  0,        0,          -1 },
	{ "Firefox",     NULL,         NULL,    1<<1,  0,        0,          -1 },
	{ "Chromium-browser", NULL,    NULL,    1<<1,  0,        0,          -1 },
	{ "Evolution",   NULL,         NULL,    1<<7,  0,        0,          -1 },
	{ NULL,          NULL,         "video", ~0,    9,        1,          -1 },
	{ NULL,          "topleft",    NULL,	0,     1,        1,          -1 },
	{ NULL,          "topcenter",  NULL,	0,     2,        1,          -1 },
	{ NULL,          "topright",   NULL,	0,     3,        1,          -1 },
	{ NULL,          "midleft",    NULL,	0,     4,        1,          -1 },
	{ NULL,          "midcenter",  NULL,	0,     5,        1,          -1 },
	{ NULL,          "midright",   NULL,	0,     6,        1,          -1 },
	{ NULL,          "botleft",    NULL,	0,     7,        1,          -1 },
	{ NULL,          "botcenter",  NULL,	0,     8,        1,          -1 },
	{ NULL,          "botright",   NULL,	0,     9,        1,          -1 },
};

/* layout(s) */
static const float mfact     = 0.50; /* factor of master area size [0.05..0.95] */
static const int nmaster     = 1;    /* number of clients in master area */
static const int resizehints = 0;    /* 1 means respect size hints in tiled resizals */

static const Layout layouts[] = {
	/* symbol     arrange function */
	{ "",       tile },    /* first entry is default */
	{ "F",      NULL },    /* no layout function means floating behavior */
	{ "M",      monocle },
};

/* key definitions */
#define MODKEY Mod1Mask
#define MODALTKEY Mod4Mask
#define TAGKEYS(KEY,TAG) \
	{ MODKEY,                       KEY,      view,           {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask,           KEY,      toggleview,     {.ui = 1 << TAG} }, \
	{ MODKEY|ShiftMask,             KEY,      tag,            {.ui = 1 << TAG} }, \
	{ MODKEY|ControlMask|ShiftMask, KEY,      toggletag,      {.ui = 1 << TAG} },


static char dmenumon[2] = "0";

#define HOME "/home/ad"
#define TERMINAL HOME "/bin/x-terminal-emulator"
#define BROWSER "ungoogled-chromium"
#define MPD_REMOTE "mini"

#define SHCMD(cmd) { .v = (const char*[]){ "/bin/sh", "-c", cmd, NULL } }
#define TERMCMD(...)   {TERMINAL, "-e", __VA_ARGS__, NULL}
#define ALTTERMCMD(...) {ALTTERMINAL, "-e", __VA_ARGS__, NULL}
#define DUPLEXCMD(cmd) {TERMINAL, "-e", "tmux", "new-session", cmd, NULL}
#define DUPLEXATTACHCMD(name, cmd) {TERMINAL, "-e", "tmux", "new-session", "-A", "-s", name, cmd, NULL}

/* commands */
static const char *termcmd[]           = TERMCMD("ksh", "-l");
static const char *termplexcmd[]       = TERMCMD("tmux");

static const char *acmecmd[]           = {"a", NULL};
static const char *dmenucmd[]          = {"dmenu_run", NULL};
static const char *searchcmd[]         = {"search", NULL};
static const char *articlesearchcmd[]  = {"articlesearch", NULL};
static const char *jabbrevcmd[]        = {"jabbrev", "-c", NULL};
static const char *unicodecmd[]        = {"unicodepick", NULL};
static const char *definecmd[]         = {"define", "--gui", NULL};
static const char *passcmd[]           = {"passmenu", "-t", "-n", NULL};
static const char *passemailcmd[]      = {"passmenu", "-t", "-u", "anders@adamsgaard.dk\t", "-n", NULL};
static const char *displaycmd[]        = {"displayselect", NULL};
static const char *displayintcmd[]     = {"mon", "int", NULL};
static const char *textcmd[]           = {"text.sh", "-i", NULL};
static const char *videocmd[]          = {"videotoggle", NULL};
static const char *screenrecordcmd[]   = {"screenrecord", NULL};
static const char *castcmd[]           = {"adcast", NULL};
static const char *plumbcmd[]          = {"clipplumb", NULL};
static const char *plumb2cmd[]         = {"clipplumb", "-c", NULL};
static const char *plumbremotecmd[]    = {"clipplumb", "-r", NULL};
static const char *plumb2remotecmd[]   = {"clipplumb", "-c", "-r", NULL};
static const char *ytdlaudiocmd[]      = {"youtube-dl-music", "-t", "-a", "-m", "-q", NULL};
static const char *showclipcmd[]       = {"showclip", NULL};
static const char *xlockcmd[]          = {"xlock", NULL};
static const char *printscreenicmd[]   = {"maimpick", NULL};
static const char *keyboardlayoutcmd[] = {"keyboard-layout-switch.sh", NULL};
static const char *torbrowsercmd[]     = {"tor-browser", NULL};
static const char *showcalcmd[]        = {"showcalendar", NULL};
static const char *mancmd[]            = {"manpick", NULL};
static const char *weathercmd[]        = {"weather", NULL};
static const char *memeselectcmd[]     = {"tardis", "-c", NULL};
static const char *contactscmd[]       = {"contacts", NULL};

static const char *browsercmd[]        = {BROWSER, HOME "/doc/pages/bookmark.html", NULL};
static const char *journalcmd[]        = TERMCMD("tmux-journal.sh");
static const char *topcmd[]            = TERMCMD("htop", "-C");
static const char *mailcmd[]           = DUPLEXCMD("mutt");
/* static const char *filecmd[]           = TERMCMD("sh", "-c", "cd ~/tmp; ls -p; $SHELL -l"); */
static const char *newscmd[]           = DUPLEXCMD("sfeed_curses $HOME/.sfeed/feeds/*");
static const char *newsrefreshcmd[]    = DUPLEXCMD("sfeed_sync");
static const char *irccmd[]            = DUPLEXATTACHCMD("irc", "irc.sh");
static const char *todocmd[]           = DUPLEXATTACHCMD("todo", "todo.sh");
static const char *calendarcmd[]       = DUPLEXATTACHCMD("calendar", "calendar.sh");
static const char *mixercmd[]          = TERMCMD("audiomixer");
static const char *musiccmd[]          = TERMCMD("ncmpc");
static const char *music_remote_cmd[]  = TERMCMD("ncmpc", "-h", MPD_REMOTE);

static const char *mpdtogglecmd[]      = {"mpc", "toggle", "-q", NULL};
static const char *mpdnextcmd[]        = {"mpc", "next", "-q", NULL};
static const char *mpdprevcmd[]        = {"mpc", "prev", "-q", NULL};
static const char *mpdstopcmd[]        = {"mpc", "stop", "-q", NULL};

static const char *mpd_remote_togglecmd[]   = {"mpc", "toggle", "-q", "-h", MPD_REMOTE, NULL};
static const char *mpd_remote_nextcmd[]     = {"mpc", "next", "-q", "-h", MPD_REMOTE, NULL};
static const char *mpd_remote_prevcmd[]     = {"mpc", "prev", "-q", "-h", MPD_REMOTE, NULL};
static const char *mpd_remote_stopcmd[]     = {"mpc", "stop", "-q", "-h", MPD_REMOTE, NULL};

static const char *mpdremotevolupcmd[]     = {"mpc", "-q", "-h", MPD_REMOTE, "volume", "+10", NULL};
static const char *mpdremotevoldncmd[]     = {"mpc", "-q", "-h", MPD_REMOTE, "volume", "-10", NULL};

static const char *audioextvolupcmd[]       = {"sndioctl", "output.level=+0.05", NULL};
static const char *audioextvoldncmd[]       = {"sndioctl", "output.level=-0.05", NULL};
static const char *audioextmutecmd[]        = {"sndioctl", "output.mute=!", NULL};

static const char *mouseleftcmd[]      = {"xdotool", "mousemove_relative", "--", "-100", "0", NULL};
static const char *mouserightcmd[]     = {"xdotool", "mousemove_relative", "--", "+100", "0", NULL};
static const char *mouseupcmd[]        = {"xdotool", "mousemove_relative", "--", "0", "-100", NULL};
static const char *mousedowncmd[]      = {"xdotool", "mousemove_relative", "--", "0", "+100", NULL};

static const char *mousefineleftcmd[]  = {"xdotool", "mousemove_relative", "--", "-10", "0", NULL};
static const char *mousefinerightcmd[] = {"xdotool", "mousemove_relative", "--", "+10", "0", NULL};
static const char *mousefineupcmd[]    = {"xdotool", "mousemove_relative", "--", "0", "-10", NULL};
static const char *mousefinedowncmd[]  = {"xdotool", "mousemove_relative", "--", "0", "+10", NULL};

static const char *mouseleftclickcmd[] = {"xdotool", "click", "1", NULL};
static const char *mouserightclickcmd[]= {"xdotool", "click", "3", NULL};

static const char *showkeyscmd[]       = {"screenkey-toggle", NULL};
static const char *goatcmd[]  = { "sh", "-c", "mpv \"$(hurl gopher://bitreich.org/0/memecache/index.meme | grep goat | grep -E '\\.(mp4|mkv|webm|avi) ' | cut -f2 -d' ' | sort -R | he ad -n 1)\"", NULL };

/* see key names in /usr/include/X11/keysymdef.h */
static Key keys[] = {
	/* modifier                     key        function        argument */
	{ MODKEY,                       XK_Return, spawn,          {.v = termplexcmd } },
	{ MODKEY|ShiftMask,             XK_Return, spawn,          {.v = termcmd } },
	{ MODKEY,                       XK_space,  spawn,          {.v = searchcmd } },
	{ MODKEY|ShiftMask,             XK_space,  spawn,          {.v = dmenucmd } },
	{ MODKEY,                       XK_grave,  spawn,          {.v = unicodecmd } },
	{ MODKEY,                       XK_slash,  spawn,          {.v = definecmd } },
	{ 0,                            XK_Print,  spawn,          SHCMD("maim ~/tmp/pic-full-$(date '+%y-%m-%d_%H:%M:%S').png") },
	{ ShiftMask,                    XK_Print,  spawn,          {.v = printscreenicmd } },
	{ MODKEY,                       XK_period, focusmon,       {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_period, tagmon,         {.i = +1 } },
	{ MODKEY,                       XK_comma,  focusmon,       {.i = -1 } },
	{ MODKEY|ShiftMask,             XK_comma,  tagmon,         {.i = -1 } },
	{ MODKEY,                       XK_Tab,    view,           {0} },
	{ MODKEY|ShiftMask,             XK_Tab,    tag,            {0} },

	{ MODKEY,                       XK_Left,   spawn,          {.v = mouseleftcmd } },
	{ MODKEY,                       XK_Right,  spawn,          {.v = mouserightcmd } },
	{ MODKEY,                       XK_Up,     spawn,          {.v = mouseupcmd } },
	{ MODKEY,                       XK_Down,   spawn,          {.v = mousedowncmd } },

	{ MODKEY|ShiftMask,             XK_Left,   spawn,          {.v = mousefineleftcmd } },
	{ MODKEY|ShiftMask,             XK_Right,  spawn,          {.v = mousefinerightcmd } },
	{ MODKEY|ShiftMask,             XK_Up,     spawn,          {.v = mousefineupcmd } },
	{ MODKEY|ShiftMask,             XK_Down,   spawn,          {.v = mousefinedowncmd } },

	{ MODKEY|ShiftMask|ControlMask, XK_Left,   spawn,          {.v = mouseleftclickcmd } },
	{ MODKEY|ShiftMask|ControlMask, XK_Right,  spawn,          {.v = mouserightclickcmd } },

	{ ShiftMask,      XF86XK_AudioRaiseVolume, spawn,          {.v = audioextvolupcmd } },
	{ ShiftMask,      XF86XK_AudioLowerVolume, spawn,          {.v = audioextvoldncmd } },
	{ ShiftMask,      XF86XK_AudioMute,        spawn,          {.v = audioextmutecmd } },

	{ ControlMask|ShiftMask, XF86XK_AudioRaiseVolume, spawn,   {.v = mpdremotevolupcmd } },
	{ ControlMask|ShiftMask, XF86XK_AudioLowerVolume, spawn,   {.v = mpdremotevoldncmd } },

	{ MODKEY,                       XK_F1,     spawn,          {.v = mancmd } },
	{ MODKEY,                       XK_F7,     spawn,          {.v = displaycmd } },
	{ MODKEY|ShiftMask,             XK_F7,     spawn,          {.v = displayintcmd } },
	{ MODKEY,                       XK_F9,     spawn,          {.v = showkeyscmd } },

	{ MODKEY,                       XK_a,      spawn,          {.v = acmecmd } },
	{ MODKEY|ShiftMask,             XK_a,      spawn,          {.v = articlesearchcmd } },
	{ MODKEY|ControlMask|ShiftMask, XK_a,      spawn,          {.v = mixercmd } },
	{ MODKEY|MODALTKEY,             XK_b,      togglebar,      {0} },
	{ MODKEY|ControlMask|ShiftMask, XK_b,      spawn,          {.v = mpdprevcmd } },
	{ MODKEY|ControlMask,           XK_b,      spawn,          {.v = mpd_remote_prevcmd } },
	{ MODKEY,                       XK_c,      spawn,          {.v = showcalcmd } },
	{ MODKEY|ShiftMask,             XK_c,      spawn,          {.v = calendarcmd } },
	{ MODKEY,                       XK_d,      spawn,          {.v = todocmd } },
	{ MODKEY|ShiftMask,             XK_d,      spawn,          {.v = journalcmd } },
	{ MODKEY,                       XK_e,      spawn,          {.v = contactscmd } },
	{ MODKEY,                       XK_f,      zoom,           {0} },
	{ MODKEY|ShiftMask,             XK_f,      togglefloating, {0} },
	{ MODKEY|ControlMask|ShiftMask, XK_g,      spawn,          {.v = goatcmd } },
	{ MODKEY,                       XK_h,      setmfact,       {.f = -0.05} },
	{ MODKEY,                       XK_i,      spawn,          {.v = showclipcmd } },
	{ MODKEY|ShiftMask,             XK_i,      spawn,          {.v = irccmd } },
	{ MODKEY,                       XK_j,      focusstack,     {.i = +1 } },
	{ MODKEY|ShiftMask,             XK_j,      incnmaster,     {.i = +1 } },
	{ MODKEY,                       XK_k,      focusstack,     {.i = -1 } },
	{ MODKEY|ShiftMask,             XK_k,      incnmaster,     {.i = -1 } },
	{ MODKEY,                       XK_l,      setmfact,       {.f = +0.05} },
	{ MODKEY|ShiftMask,             XK_l,      setlayout,      {.v = &layouts[1]} },
	{ MODKEY|ControlMask|ShiftMask, XK_l,      spawn,          {.v = keyboardlayoutcmd } },
	{ MODKEY,                       XK_m,      spawn,          {.v = mailcmd } },
	{ MODKEY|ShiftMask,             XK_m,      setlayout,      {.v = &layouts[2]} },
	{ MODKEY|ControlMask|ShiftMask, XK_m,      spawn,          {.v = musiccmd } },
	{ MODKEY|ControlMask,           XK_m,      spawn,          {.v = music_remote_cmd } },
	{ MODKEY,                       XK_n,      spawn,          {.v = newscmd } },
	{ MODKEY|ShiftMask,             XK_n,      spawn,          {.v = newsrefreshcmd } },
	{ MODKEY|ControlMask|ShiftMask, XK_n,      spawn,          {.v = mpdnextcmd } },
	{ MODKEY|ControlMask,           XK_n,      spawn,          {.v = mpd_remote_nextcmd } },
	{ MODKEY,                       XK_o,      spawn,          {.v = castcmd } },
	{ MODKEY|ShiftMask,             XK_o,      spawn,          {.v = topcmd } },
	{ MODKEY,                       XK_p,      spawn,          {.v = passcmd } },
	{ MODKEY|ShiftMask,             XK_p,      spawn,          {.v = passemailcmd } },
	{ MODKEY|ControlMask|ShiftMask, XK_p,      spawn,          {.v = mpdtogglecmd } },
	{ MODKEY|ControlMask,           XK_p,      spawn,          {.v = mpd_remote_togglecmd } },
	{ MODKEY,                       XK_q,      killclient,     {0} },
	{ MODKEY|ShiftMask,             XK_q,      quit,           {0} },
	{ MODKEY,                       XK_r,      spawn,          {.v = screenrecordcmd } },
	{ MODKEY|ShiftMask,             XK_s,      spawn,          {.v = memeselectcmd } },
	{ MODKEY|ControlMask|ShiftMask, XK_s,      spawn,          {.v = mpdstopcmd } },
	{ MODKEY|ControlMask,           XK_s,      spawn,          {.v = mpd_remote_stopcmd } },
	{ MODKEY,                       XK_t,      spawn,          {.v = textcmd } },
	{ MODKEY|ShiftMask,             XK_t,      setlayout,      {.v = &layouts[0]} },
	{ MODKEY,                       XK_u,      spawn,          {.v = plumbcmd } },
	{ MODKEY|ShiftMask,             XK_u,      spawn,          {.v = plumb2cmd } },
	{ MODKEY|ControlMask,           XK_u,      spawn,          {.v = plumbremotecmd } },
	{ MODKEY|ControlMask|ShiftMask, XK_u,      spawn,          {.v = plumb2remotecmd } },
	{ MODKEY,                       XK_v,      spawn,          {.v = videocmd } },
	{ MODKEY,                       XK_w,      spawn,          {.v = browsercmd } },
	{ MODKEY|ShiftMask,             XK_w,      spawn,          {.v = torbrowsercmd } },
	{ MODKEY|ControlMask|ShiftMask, XK_w,      spawn,          {.v = weathercmd } },
	{ MODKEY,                       XK_z,      spawn,          {.v = xlockcmd } },

	{ MODKEY,                       XK_0,      view,           {.ui = ~0 } },
	{ MODKEY|ShiftMask,             XK_0,      tag,            {.ui = ~0 } },
	TAGKEYS(                        XK_1,                      0)
	TAGKEYS(                        XK_2,                      1)
	TAGKEYS(                        XK_3,                      2)
	TAGKEYS(                        XK_4,                      3)
	TAGKEYS(                        XK_5,                      4)
	TAGKEYS(                        XK_6,                      5)
	TAGKEYS(                        XK_7,                      6)
	TAGKEYS(                        XK_8,                      7)
	TAGKEYS(                        XK_9,                      8)
};

/* button definitions */
/* click can be ClkTagBar, ClkLtSymbol, ClkStatusText, ClkWinTitle, ClkClientWin, or ClkRootWin */
static Button buttons[] = {
	/* click                event mask      button          function        argument */
	{ ClkLtSymbol,          0,              Button1,        setlayout,      {0} },
	{ ClkLtSymbol,          0,              Button3,        setlayout,      {.v = &layouts[2]} },
	{ ClkWinTitle,          0,              Button2,        zoom,           {0} },
	{ ClkStatusText,        0,              Button2,        spawn,          {.v = termcmd } },
	{ ClkClientWin,         MODKEY,         Button1,        movemouse,      {0} },
	{ ClkClientWin,         MODKEY,         Button2,        togglefloating, {0} },
	{ ClkClientWin,         MODKEY,         Button3,        resizemouse,    {0} },
	{ ClkTagBar,            0,              Button1,        view,           {0} },
	{ ClkTagBar,            0,              Button3,        toggleview,     {0} },
	{ ClkTagBar,            MODKEY,         Button1,        tag,            {0} },
	{ ClkTagBar,            MODKEY,         Button3,        toggletag,      {0} },
};
